% This script computes the cost-minimizing contract with and without a
% concavity constraint, and it generates a plot with output, x, on the 
% x-axis, and the transfer that the contract stipulates in utils, v(x), 
% on the y-axis.
% To execute, you must first download and install the CVX package from 
% http://cvxr.com/cvx/download/

clc
clear all

% Output x takes values in [yL, yH]
yL = -50;
yH = 100;

% PDF of Output: Enter your favorite pdf of output. It must be of the form
% f(x,a), where 'x' is output and 'a' is effort. 

% The following is an FGM copula pdf that  has expectation a.  
beta = @(a) ( (3.*a - 2.*((yH.^3 - yL.^3)./(yH.^2 - yL.^2)))./(1.5.*(yH.^2 - yL.^2) - 2.*((yH.^3 - yL.^3)./(yL+yH))));
alph = @(a) ( 2.*(1-beta(a).*(yH-yL))./(yH.^2 - yL.^2));
f = @(x,a) (alph(a).*x + beta(a));
% Must check manually that f(x,a)>=0 for all x and a in range of interest

% Check that the score function f_a/f is increasing in x.
%fa = @(x,a) ((f(x,a+0.01)-f(x,a))./0.01);
%l = @(x,a) (fa(x,a)./f(x,a));
%plot(X,l(X,1),'r',X,l(X,10),'b',X,l(X,20),'c',X,l(X,30),'g',X,l(X,40),'y',X,l(X,49),'k','LineWidth',3)

% The agent's utility exhibits % CRRA with coefficient 1-rho.
rho = 0.8; % 1- the coefficient of RRA
u = @(om) ( (om.^rho)./rho ); % Utility function
uinv = @(z) ( (rho.*z).^(1./rho) ); % Inverse utility function

% We assume that the agent has an isoelastic cost function
c0 = 0.02; % Multiplier
p = 1; % 1 / Elasticitity

cost = @(A) ((c0./(p+1)).*(A.^(p+1)));
cp = @(A) (c0.*(A.^p)); % Isoelastic effort cost with intercept c0 and elasticitiy 1/p

% Other parameters
u0 = 50; % Reservation utility of the agent
%wmin = 20; % Limited liability (in $)
umin = 0; % Limited liability (in utils)

% We approximate the integrals using the trapezoidal rule. 
N = 1000; % Size of the discrete grid on which the integrals are evaluated
X = linspace(yL,yH,N)'; % Defines the grid of x 
cc = (max(X)-min(X))/N;

% Fix the principal's target effort level
a_target = 33;

% For convenience, define the PDF is a vector (instead of a function)
ff = f(X,a_target);

% Define f_a. We use an approximation from ff so that it is easier to
% change the PDF
f_a = (f(X,a_target+0.01)-f(X,a_target))./0.01;

% First, we solve for the optimal contract WITHOUT the concavity constraint
cvx_begin
    variables v(N)
    dual variables lam mu
    minimize( cc.*sum( ((rho.*v).^(1./rho)).*ff) )
    subject to
        mu : cc.*sum(v.*f_a) >= cp(a_target);
        lam : cc.*sum(v.*ff) - cost(a_target) >= u0;
        v >= umin;
cvx_end

% Compute optimal contract (in utils and $) and the associated cost
v_noconc = v;
w_noconc = uinv(v_noconc);
Cost_noconc = trapz(X,w_noconc.*ff);

% We now solve for the optimal contract WITH the concavity constraint
cvx_begin
    variables v2(N)
    dual variables lam2 mu2 nu2
    minimize( cc.*sum( ((rho.*v2).^(1./rho)).*ff) )
    subject to
        mu2 : cc.*sum(v2.*f_a) >= cp(a_target);
        lam2 : cc.*sum(v2.*ff) - cost(a_target) >= u0;        
        v2 >= umin;
        nu2 : 2.*v2(2:end-1) >= v2(1:end-2) + v2(3:end);
cvx_end

% Compute optimal contract (in utils and $) and the associated cost
v_conc = v2;
w_conc = uinv(v_conc);
Cost_conc = trapz(X,w_conc.*ff);

% Plot the two contracts (in utils) on the same graph
figure('units','normalized','outerposition',[0 0 1 1])
plot(X,v_noconc,'b', X,v_conc,'r', 'LineWidth',3)
xlabel('$x$','FontSize',50,'interpreter','latex'); ylabel('$v(x)$','FontSize',50,'interpreter','latex')
set(gca,'FontSize',20)
legend({'"HM Contract"','"No-Gaming Contract"'},'Location','northwest','FontSize',30);
legend('boxoff')
title(['Assumptions: Constant coefficient of RRA=',num2str(1-rho), ', $a$=',num2str(a_target),', $c_0$=',num2str(c0),', $p$=',num2str(p),', $u_0$ =',num2str(u0),' and $\underline{u}$=',num2str(umin)],'FontSize',30,'FontWeight','normal','interpreter','latex')
fig = gcf;
fig.Color = 'w';
