% This script computes the profit-maximizing effort and the corresponding
% optimal contract with and without a concavity constraint.
% To execute, you must first download and install the CVX package from 
% http://cvxr.com/cvx/download/

clc
%clear all

% Output x takes values in [yL, yH]
yL = -50;
yH = 100;

% PDF of Output: Enter your favorite pdf of output. It must be of the form
% f(x,a), where 'x' is output and 'a' is effort. 

% The following is an FGM copula pdf that has expectation a.  
beta = @(a) ( (3.*a - 2.*((yH.^3 - yL.^3)./(yH.^2 - yL.^2)))./(1.5.*(yH.^2 - yL.^2) - 2.*((yH.^3 - yL.^3)./(yL+yH))));
alph = @(a) ( 2.*(1-beta(a).*(yH-yL))./(yH.^2 - yL.^2));
f = @(x,a) (alph(a).*x + beta(a));
% Must check manually that f(x,a)>=0 for all x and a in the range of interest

% The agent's utility exhibits CRRA with coefficient 1-rho.
rho = 0.5; % 1- the coefficient of RRA
u = @(om) ( (om.^rho)./rho ); % Utility function
uinv = @(z) ( (rho.*z).^(1./rho) ); % Inverse utility function

% We assume that the agent has an isoelastic cost function
c0 = 0.01; % Multiplier
p = 1; % 1 / Elasticitity

cost = @(A) ((c0./(p+1)).*(A.^(p+1)));
cp = @(A) (c0.*(A.^p)); % Isoelastic effort cost with intercept c0 and elasticitiy 1/p

%cost = @(A) (c0.*(A+0.5.*(A.^2))); 
%cp = @(A) (c0.*(1+A)); % Alternative cost functions with c'(0)>0

% Other parameters
u0 = 0; % Reservation utility of the agent
wmin = 20; % Limited liability (in $)
umin = u(wmin); % Limited liability (in utils)

m = 1; % Principal's gross profit margin (If output is x, then gross profit = mx)

% We approximate the integrals using the Riemann sums. 
N = 1001; % Length of the discrete grid on which the integrals are evaluated
X = linspace(yL,yH,N)'; % Defines the grid of x 
cc = (max(X)-min(X))/N;

% Computes the cost-minimizing contract for every feasible effort level
AA = linspace(0,5,101); % Grid of target effort levels

for j=1:101

% Fix the principal's target effort level for the iteration
a_target = AA(j);

% For convenience, define the PDF is a vector (instead of a function)
ff = f(X,a_target);

% Define f_a. We use an approximation from ff so that it is easier to
% change the PDF
f_a = (f(X,a_target+0.01)-f(X,a_target))./0.01;

% First, we solve for the optimal contract WITHOUT the concavity constraint
cvx_begin
    variables v(N)
    minimize( cc.*sum( ((rho.*v).^(1./rho)).*ff) )
    subject to
        cc.*sum(v.*f_a) >= cp(a_target);
        cc.*sum(v.*ff) - cost(a_target) >= u0;
        v >= umin;
cvx_end

% Compute optimal contract (in utils and $) and the associated cost
v_noconc(:,j) = v;
w_noconc(:,j) = uinv(v_noconc(:,j));
Profit_noconc(j) = m*a_target - trapz(X,w_noconc(:,j).*ff);

% We now solve for the optimal contract WITH the concavity constraint
cvx_begin
    variables v2(N)
    minimize( cc.*sum( ((rho.*v2).^(1./rho)).*ff) )
    subject to
        cc.*sum(v2.*f_a) >= cp(a_target);
        cc.*sum(v2.*ff) - cost(a_target) >= u0;        
        v2 >= umin;
        2.*v2(2:end-1) >= v2(1:end-2) + v2(3:end);
cvx_end

% Compute optimal contract (in utils and $) and the associated cost
v_conc(:,j) = v2;
w_conc(:,j) = uinv(v_conc(:,j));
Profit_conc(j) = m*a_target - trapz(X,w_conc(:,j).*ff);

end

% Find the profit-maximizing effort for each case
[PrMax_noconc j_noconc] = max(Profit_noconc);
aMax_noconc = AA(j_noconc);
v_opt_noconc = v_noconc(:,j_noconc);
u_opt_noconc = trapz(X, v_opt_noconc.*f(X,aMax_noconc)) - cost(aMax_noconc);

[PrMax_conc j_conc] = max(Profit_conc);
aMax_conc = AA(j_conc);
v_opt_conc = v_conc(:,j_conc);
u_opt_conc = trapz(X, v_opt_conc.*f(X,aMax_conc)) - cost(aMax_conc);

% Plot the two contracts (in utils) on the same graph
figure('units','normalized','outerposition',[0 0 1 1])
%subplot(1,2,1); 
plot(X,v_opt_noconc,'b', X,v_opt_conc,'r', 'LineWidth',3)
xlabel('$y$','FontSize',50,'interpreter','latex'); ylabel('$v(y)$','FontSize',50,'interpreter','latex')
set(gca,'FontSize',20)
legend({['"H-M Contract", $a^*=$',num2str(aMax_noconc), '$, U_A=$',num2str(u_opt_noconc)],['"No Gaming Contract", $a^*=$',num2str(aMax_conc), '$, U_A=$',num2str(u_opt_conc)]},'Location','northwest','FontSize',30,'interpreter','latex');
legend('boxoff')
title(['Assumptions: Constant coefficient of RRA=',num2str(1-rho), ', $m$=',num2str(m),', $c_0$=',num2str(c0),', $p$=',num2str(p),', $u_0$=',num2str(u0),' and $\underline{w}$=',num2str(wmin)],'FontSize',30,'FontWeight','normal','interpreter','latex')
fig = gcf;
fig.Color = 'w';

%hold on